@@ -1,4 +1,4 @@
-# Copyright (c) 2017-2018 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2017-2019 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -97,10 +97,10 @@ class ToolbarTitle(Gtk.Bin):
         self.__signal_id = None
         self.__entry_changed_id = None
         self.__suggestion_id = None
+        self.__password_timeout_id = None
         self.__secure_content = True
         self.__size_allocation_timeout = Type.NONE  # CSS update needed
         self.__width = -1
-        self.__uri = ""
         self.__cancellable = Gio.Cancellable.new()
         self.__dns_suffixes = ["com", "org"]
         for string in reversed(GLib.get_language_names()):
@@ -127,6 +127,7 @@ class ToolbarTitle(Gtk.Bin):
         self.__popover = UriPopover(window)
         self.__popover.set_relative_to(self.__entry)
         self.__popover.connect("closed", self.__on_popover_closed)
+        self.__credentials_popover = None
         # Reload/Stop
         self.__action_image1 = builder.get_object("action_image1")
         # Bookmarks/Clear
@@ -225,18 +226,17 @@ class ToolbarTitle(Gtk.Bin):
         self.__signal_id = self.__entry.connect("changed",
                                                 self.__on_entry_changed)
 
-    def set_uri(self, uri):
+    def update(self):
         """
-            Update internal uri
+            Update toolbar based on current webview
             @param text as str
         """
-        if uri == self.__uri:
-            return
-        self.__uri = uri
+        uri = self.__window.container.current.webview.uri
         self.set_tooltip_text(uri)
         self.__input_warning_shown = False
         self.__secure_content = True
-        self.__update_secure_content_indicator()
+        if self.__password_timeout_id is None:
+            self.__update_secure_content_indicator()
         bookmark_id = App().bookmarks.get_id(uri)
         if bookmark_id is not None:
             icon_name = "starred-symbolic"
@@ -253,7 +253,8 @@ class ToolbarTitle(Gtk.Bin):
         self.__window.set_title(title)
         markup = False
         # Do not show this in titlebar
-        parsed = urlparse(self.__uri)
+        uri = self.__window.container.current.webview.uri
+        parsed = urlparse(uri)
         if parsed.scheme in ["populars", "about"]:
             self.__set_default_placeholder()
             return
@@ -263,7 +264,7 @@ class ToolbarTitle(Gtk.Bin):
             else:
                 self.__placeholder.set_text(title)
         else:
-            self.__placeholder.set_text(self.__uri)
+            self.__placeholder.set_text(uri)
         if not self.__popover.is_visible():
             self.__placeholder.set_opacity(0.8)
             self.set_text_entry("")
@@ -358,18 +359,26 @@ class ToolbarTitle(Gtk.Bin):
             @param form_uri as str
             @param page_id as int
         """
+        if self.__password_timeout_id is not None:
+            GLib.source_remove(self.__password_timeout_id)
+        self.__entry.set_icon_from_icon_name(Gtk.EntryIconPosition.PRIMARY,
+                                             "dialog-password-symbolic")
+        self.__entry.set_icon_tooltip_text(Gtk.EntryIconPosition.PRIMARY,
+                                           _("Save password for: %s") % uri)
+        self.__password_timeout_id = GLib.timeout_add(
+            10000, self.__update_secure_content_indicator)
+        if self.__credentials_popover is not None:
+            self.__credentials_popover.destroy()
         from eolie.popover_credentials import CredentialsPopover
-        popover = CredentialsPopover(uuid, user_form_name, user_form_value,
+        self.__credentials_popover = CredentialsPopover(
+                                     uuid,
+                                     user_form_name,
+                                     user_form_value,
                                      pass_form_name,
                                      uri,
                                      form_uri,
                                      page_id,
                                      self.__window)
-        popover.set_relative_to(self.__entry)
-        popover.set_pointing_to(self.__entry.get_icon_area(
-            Gtk.EntryIconPosition.PRIMARY))
-        popover.connect("closed", self.__on_popover_closed)
-        popover.popup()
 
     def show_indicator(self, indicator):
         """
@@ -407,14 +416,6 @@ class ToolbarTitle(Gtk.Bin):
         if not self.__entry.is_focus():
             self.__entry.grab_focus()
 
-    @property
-    def uri(self):
-        """
-            Get current uri
-            @return str
-        """
-        return self.__uri
-
     @property
     def progress(self):
         """
@@ -433,11 +434,17 @@ class ToolbarTitle(Gtk.Bin):
             @param icon_pos as Gtk.EntryIconPosition
             @param event as Gdk.Event
         """
+        def on_popover_closed(popover):
+            self.__credentials_popover = None
+
         if self.__entry.get_icon_name(Gtk.EntryIconPosition.PRIMARY) ==\
-                "edit-copy-symbolic":
-            Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD).set_text(
-                self.__uri,
-                -1)
+                "dialog-password-symbolic":
+            self.__update_secure_content_indicator()
+            self.__credentials_popover.set_relative_to(self.__entry)
+            self.__credentials_popover.set_pointing_to(
+                self.__entry.get_icon_area(Gtk.EntryIconPosition.PRIMARY))
+            self.__credentials_popover.connect("closed", on_popover_closed)
+            self.__credentials_popover.popup()
 
     def _on_enter_notify(self, widget, event):
         """
@@ -447,13 +454,10 @@ class ToolbarTitle(Gtk.Bin):
         """
         if self.__popover.is_visible():
             return
-        parsed = urlparse(self.__uri)
+        uri = self.__window.container.current.webview.uri
+        parsed = urlparse(uri)
         if parsed.scheme in ["http", "https", "file"]:
-            self.__entry.set_icon_from_icon_name(Gtk.EntryIconPosition.PRIMARY,
-                                                 "edit-copy-symbolic")
-            self.__entry.set_icon_tooltip_text(Gtk.EntryIconPosition.PRIMARY,
-                                               _("Copy address"))
-            self.set_text_entry(self.__uri)
+            self.set_text_entry(uri)
         else:
             self.__set_default_placeholder()
 
@@ -483,12 +487,13 @@ class ToolbarTitle(Gtk.Bin):
         webview = self.__window.container.current.webview
         self.__action_image2.set_from_icon_name("edit-clear-symbolic",
                                                 Gtk.IconSize.MENU)
-        parsed = urlparse(self.__uri)
+        uri = self.__window.container.current.webview.uri
+        parsed = urlparse(uri)
         value = webview.get_current_text_entry()
         if value:
             self.set_text_entry(value)
         elif parsed.scheme in ["http", "https", "file"]:
-            self.set_text_entry(self.__uri)
+            self.set_text_entry(uri)
             self.__placeholder.set_opacity(0)
         else:
             self.__set_default_placeholder()
@@ -579,15 +584,20 @@ class ToolbarTitle(Gtk.Bin):
                 GLib.idle_add(self.__window.close_popovers)
                 parsed = urlparse(uri)
                 # Search a missing scheme
-                if uri.find(".") != -1 and not parsed.scheme:
-                    db_uri = App().history.get_match(uri)
-                    if db_uri is not None:
-                        db_parsed = urlparse(db_uri)
-                        if db_parsed.netloc.startswith("www.") and\
-                                not parsed.netloc.startswith("www."):
-                            uri = "%s://www.%s" % (db_parsed.scheme, uri)
-                        else:
-                            uri = "%s://%s" % (db_parsed.scheme, uri)
+                if uri.find(".") != -1 and\
+                        uri.find(" ") == -1 and\
+                        not parsed.scheme:
+                    # Add missing www.
+                    if not uri.startswith("www."):
+                        db_uri = App().history.get_match("://www." + uri)
+                        if db_uri is not None:
+                            uri = "www." + uri
+                    # Add missing scheme
+                    db_uri = App().history.get_match("https://" + uri)
+                    if db_uri is None:
+                        uri = "http://" + uri
+                    else:
+                        uri = "https://" + uri
                 self.__window.container.load_uri(uri)
                 self.__window.container.set_expose(False)
                 if self.__entry_changed_id is not None:
@@ -734,7 +744,6 @@ class ToolbarTitle(Gtk.Bin):
         self.__completion_model.clear()
         self.__placeholder.set_opacity(0.8)
         self.set_text_entry("")
-        self.__update_secure_content_indicator()
         uri = view.webview.uri
         if uri is not None:
             bookmark_id = App().bookmarks.get_id(uri)
@@ -749,9 +758,9 @@ class ToolbarTitle(Gtk.Bin):
         """
             Leave widget
         """
-        self.__update_secure_content_indicator()
         self.__placeholder.set_opacity(0.8)
-        parsed = urlparse(self.__uri)
+        uri = self.__window.container.current.webview.uri
+        parsed = urlparse(uri)
         if parsed.scheme in ["http", "https", "file"]:
             self.set_text_entry("")
         else:
@@ -775,7 +784,9 @@ class ToolbarTitle(Gtk.Bin):
         """
             Update PRIMARY icon, Gtk.Entry should be set
         """
-        parsed = urlparse(self.__uri)
+        self.__password_timeout_id = None
+        uri = self.__window.container.current.webview.uri
+        parsed = urlparse(uri)
         if (parsed.scheme == "https" and self.__secure_content) or\
                 parsed.scheme == "file":
             self.__entry.set_icon_from_icon_name(
@@ -891,7 +902,7 @@ class ToolbarTitle(Gtk.Bin):
             @param popover as Gtk.popover
         """
         self.__cancellable.cancel()
-        self.__cancellable.reset()
+        self.__cancellable = Gio.Cancellable.new()
         self.__completion_model.clear()
         webview = self.__window.container.current.webview
         if popover == self.__popover:
@@ -924,7 +935,8 @@ class ToolbarTitle(Gtk.Bin):
                 return
         parsed = urlparse(value)
         is_uri = parsed.scheme in ["about, http", "file", "https", "populars"]
-        parsed = urlparse(self.__uri)
+        uri = self.__window.container.current.webview.uri
+        parsed = urlparse(uri)
         if value:
             self.__placeholder.set_opacity(0)
             # We are doing a search, show popover
@@ -955,7 +967,7 @@ class ToolbarTitle(Gtk.Bin):
         task_helper.run(self.__populate_completion, value)
 
         self.__cancellable.cancel()
-        self.__cancellable.reset()
+        self.__cancellable = Gio.Cancellable.new()
 
         network = Gio.NetworkMonitor.get_default().get_network_available()
         parsed = urlparse(value)
