@@ -1,4 +1,4 @@
-# Copyright (c) 2017-2018 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2017-2019 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -132,7 +132,7 @@ class Window(Gtk.ApplicationWindow):
         """
         uri = webview.uri
         if uri is not None:
-            self.toolbar.title.set_uri(uri)
+            self.toolbar.title.update()
             accept_tls = App().websettings.get_accept_tls(uri)
             self.toolbar.end.show_tls_button(accept_tls)
         if webview.popups:
@@ -419,6 +419,11 @@ class Window(Gtk.ApplicationWindow):
             self.__container.try_close_view(self.container.current)
         elif string == "reload":
             self.container.current.webview.reload()
+        elif string == "settings":
+            # Rework all this code to use actions like in Lollypop
+            from eolie.settings import SettingsDialog
+            dialog = SettingsDialog(self)
+            dialog.show()
         elif string == "home":
             self.container.current.webview.load_uri(App().start_page)
         elif string == "source":
